#!/bin/bash
# ${copyright}
# ------------------------------------------------------------------------------
# # Install collectd and some dependencies for plugins used in collectd.conf
# Run this script as root

trap "echo 'Installation process canceled by user.' && exit" SIGINT

if [ -n "$SPLUNK_URL" ]; then
    splunk_url=$SPLUNK_URL
fi

if [ -n "$HEC_PORT" ]; then
    hec_port=$HEC_PORT
fi

if [ -n "$UDP_PORT" ]; then
    udp_port=$UDP_PORT
fi

if [ -n "$HEC_TOKEN" ]; then
    hec_token=$HEC_TOKEN
fi

if [ -n "$DIMENSIONS" ]; then
    dimension=$DIMENSIONS
fi

if [ -n "$INTERVAL" ]; then
    interval=$INTERVAL
fi

if [ -n "$METRIC_TYPES" ]; then
    metric_type=$METRIC_TYPES
fi

if [ -n "$METRIC_OPTS" ]; then
    metric_opt=$METRIC_OPTS
fi

if [ -n "$METRIC_BUFFER_SIZE" ]; then
    metric_buffer_size=$METRIC_BUFFER_SIZE
fi

if [ -n "$METRIC_USE_UDP" ]; then
    metric_use_udp=$METRIC_USE_UDP
fi

if [ -n "$AUTHENTICATED_INSTALL" ]; then
    authenticated_install=$AUTHENTICATED_INSTALL
fi

if [ -n "$ITSI_ENABLE_DOCKER" ]; then
    enable_docker=$ITSI_ENABLE_DOCKER
fi

if [ -n "$ITSI_DOCKER_SOCK" ]; then
    docker_unix_sock=$ITSI_DOCKER_SOCK
fi

if [[ $metric_use_udp = "YES" ]] && ( [[ ! "$splunk_url" ]] || [[ ! $udp_port ]] ); then
    printf "\033[31mMissing required arguments - SPLUNK_URL and UDP_PORT.\033[0m\n"
    exit 1;
fi

if [[ $metric_use_udp != "YES" ]] && ( [[ ! "$splunk_url" ]] || [[ ! $hec_port ]] || [[ ! $hec_token ]] ); then
    printf "\033[31mMissing required arguments - SPLUNK_URL, HEC_PORT and HEC_TOKEN.\033[0m\n"
    exit 1;
fi

KNOWN_DISTRIBUTION="(Debian|Ubuntu|RedHat|CentOS)"
DISTRIBUTION=$(lsb_release -d 2>/dev/null | grep -Eo $KNOWN_DISTRIBUTION)
SII_VERSION="2.2.5"
COLLECTD_VERSION="5\\.([7-9]|10|11|12)\\.[0-9]"
COLLECTD_VERSION_DARWIN="5\\.([7-9]|10|11|12)\\.[0-9]\\S*"
UNSUPP_COLLECTD_INSTALLED_MSG='Unsupported version of collectd already installed. Supported versions: 5.7.x - 5.12.x'
UNSUPP_COLLECTD_AVAILABLE_MSG='Failed to install supported version of collectd. Require collectd version 5.7.x - 5.12.x'

# Detect OS / Distribution
if [ -f /etc/debian_version -o "$DISTRIBUTION" == "Debian" -o "$DISTRIBUTION" == "Ubuntu" ]; then
    OS="Debian"
elif [ -f /etc/redhat-release -o "$DISTRIBUTION" == "RedHat" -o "$DISTRIBUTION" == "CentOS" ]; then
    OS="RedHat"
elif [[ -f /etc/os-release && $(grep "^NAME" /etc/os-release | grep -Eo '".*"' | tr -d \") =~ "Amazon Linux" ]]; then
    OS="EC2"
elif [[ -f /etc/os-release && $(grep "^NAME" /etc/os-release | grep -Eo '".*"' | tr -d \") =~ "openSUSE" ]]; then
    OS="OpenSuse"
elif [[ -f /etc/os-release && $(grep "^PRETTY_NAME" /etc/os-release | grep -Eo '".*"' | tr -d \") =~ "SUSE Linux Enterprise" ]]; then
    OS="EnterpriseSuse"
else
    OS=$(uname -s)
fi

# Detect root user
if [ $(echo "$UID") = "0" ]; then
    _sudo=''
else
    _sudo='sudo'
fi

# Get dimension list
if [ -n "$dimension" ]; then
    IFS=',' read -a dimensions <<< "$dimension"
fi

# Get metric sources list
if [ -n "$metric_type" ]; then
    IFS=',' read -a metric_types <<< "$metric_type"
fi

# Get metric options list
if [ -n "$metric_opt" ]; then
    IFS=',' read -a metric_opts <<< "$metric_opt"
fi

function print_info {
    echo -e "\033[32m\n $* \n\033[0m"
}

function print_warning {
    echo -e "\033[31m\n $* \n\033[0m"
}

function sed_inplace {
    if [ "$OS" = "Darwin" ]; then
        sed -i '' "$1" "$2"
    elif [ "$OS" = "SunOS" ]; then
        sed "$1" "$2" > "${2}.tmp" && mv "${2}.tmp" "$2"
    else
        sed -i "$1" "$2"
    fi
}

function configure {
  setAttributesByCloud
  print_info "Step:Configure agent..."
  update_conf

  # configure collectd
  $_sudo mkdir -p /etc/collectd/

  case "$collectd_version" in
    5.7*|5.8*)
        plugin_dir=plugin_5_7_5_8
        ;;
    5.9*|5.10*|5.11*|5.12*)
        plugin_dir=plugin_5_9_5_10
        ;;
    *)
        print_warning "Error while installing Splunk plugins. Exiting .."
        exit 1
        ;;
  esac

  # copy collectd.conf and the plugins to the appropriate directories
  case "$OS" in
    RedHat|EC2|OpenSuse|EnterpriseSuse)
        $_sudo cp collectd.conf /etc/
        $_sudo cp "${plugin_dir}/write_splunk.so" /usr/lib64/collectd/
        $_sudo cp "${plugin_dir}/docker.so" /usr/lib64/collectd/
        $_sudo cp "${plugin_dir}/processmon.so" /usr/lib64/collectd/
        ;;
    Darwin)
        collectd_version=$(brew ls --versions collectd | grep -Eo "$COLLECTD_VERSION_DARWIN" | head -1)
        brew_prefix=$(get_brew_prefix)
        $_sudo cp collectd.conf "$brew_prefix/Cellar/collectd/${collectd_version}/etc/"
        $_sudo cp "${plugin_dir}/write_splunk.so" "$brew_prefix/Cellar/collectd/${collectd_version}/lib/collectd/"
        $_sudo cp "${plugin_dir}/docker.so" "$brew_prefix/Cellar/collectd/${collectd_version}/lib/collectd/"
        ;;
    SunOS)
        $_sudo cp collectd.conf /etc/opt/csw/
        $_sudo cp "${plugin_dir}/write_splunk-solaris.so" "/opt/csw/lib/collectd/write_splunk.so"
        ;;
    *)
        $_sudo cp collectd.conf /etc/collectd/
        if [ "$debian_libcurl3" = true ] || [ "$plugin_dir" = "plugin_5_9_5_10" ]; then
            $_sudo cp "${plugin_dir}/write_splunk.so" /usr/lib/collectd/
            $_sudo cp "${plugin_dir}/docker.so" /usr/lib/collectd/
            $_sudo cp "${plugin_dir}/processmon.so" /usr/lib/collectd/
        elif [ "$debian_libcurl4" = true ]; then
            $_sudo cp "${plugin_dir}/deb_libcurl4/write_splunk.so" /usr/lib/collectd/
            $_sudo cp "${plugin_dir}/deb_libcurl4/docker.so" /usr/lib/collectd/
            $_sudo cp "${plugin_dir}/deb_libcurl4/processmon.so" /usr/lib/collectd/
        else
            print_warning "Error while installing splunk plugins. exiting .."
            exit 1
        fi
        ;;
  esac

  command='start'
  if hash pgrep 2>/dev/null && pgrep -x collectd >/dev/null; then
    command='restart';
  fi

  if [ "$OS" = "Darwin" ]; then
    $_sudo brew services "$command" collectd
  elif [ "$OS" = "SunOS" ]; then
    $_sudo svcadm enable cswcollectd
  else
    $_sudo service collectd "$command"
  fi
}

function enable_collectd {
    if [ -x "$(command -v systemctl)" ]; then
        $_sudo systemctl enable collectd
    fi
}

function check_selinux {
    # check if system is SELinux and if yes check if it is enabled
    if ! hash getenforce 2>/dev/null || [[ $(getenforce) != "Enforcing" ]]; then
        return
    else
        docs_link="https://docs.splunk.com/Documentation/InfraApp/${SII_VERSION}/Admin/SELinux"
        print_warning "################################################################################\n"\
        "SELinux is currently enabled and may cause data collection to fail.\n"\
        "To update your settings to allow for data collection see: $docs_link\n"\
        "################################################################################"
    fi

    # SELinux is enabled. Check if --force continue or Ask if user wants to continue or not
    if [[ $1 = "--force-continue" ]]; then
        print_info "Continuing Force Installation..."
    else
        print_info "Use \"./install_agent --force-continue\" to force installation without prompt."
        read -p "Continue Installation (y/n)? " yn
        case $yn in
            [Yy]* )
                print_info "Continuing Installation..."
                ;;
            * )
                print_warning "Cancelling Installation..."
                exit 1
                ;;
        esac
    fi
}

function get_splunk_hostname {
    splunk_hostname=$(hostname)
    splunk_home="/opt/splunkforwarder"
    if [ "$OS" = "Darwin" ]; then
        splunk_home="/Applications/splunkforwarder"
    fi

    inputs_conf="$splunk_home/etc/system/local/inputs.conf"
    if [ -f "$inputs_conf" ]; then
        inputs_conf_host=$($_sudo cat "$inputs_conf" | grep 'host' | cut -d '=' -f2 | tr -d '[:space:]')
        if [ -n "$inputs_conf_host" ]; then
          splunk_hostname="$inputs_conf_host"
        fi
    fi
    if [ -z "$inputs_conf_host" ]; then
        print_info "Could not find hostname used by Splunk - Using $splunk_hostname by default..."
    fi
}

function update_conf {
    for s in "${metric_types[@]}"
    do
        # processmon not available for Solaris
        if [ "$s" = "processmon" ] && [ "$OS" = "SunOS" ]; then
            continue
        fi
        sed_inplace "s/#LoadPlugin $s/LoadPlugin $s/g" "collectd.conf"
    done
    for s in "${metric_opts[@]}"
    do
        if [ $s = "cpu.by_cpu" ]; then
            sed_inplace "s/ReportByCpu false/ReportByCpu true/g" "collectd.conf"
        fi
    done
    # Add custom plugin to collectd.conf
    echo -e "##############################################################################" >> collectd.conf
    echo -e "# Customization for Splunk                                                   #" >> collectd.conf
    echo -e "#----------------------------------------------------------------------------#" >> collectd.conf
    echo -e "# This plugin sends all metrics data from other plugins to Splunk via HEC.   #" >> collectd.conf
    echo -e "##############################################################################" >> collectd.conf
    echo -e "\n<Plugin write_splunk>" >> collectd.conf
    echo -e "           server \"$splunk_url\"" >> collectd.conf
    if [ "$metric_use_udp" = "YES" ]; then
        echo -e "           useudp true" >> collectd.conf
        echo -e "           udpport $udp_port" >> collectd.conf
        if [ -n "$metric_buffer_size" ]; then
            echo -e "           buffersize $metric_buffer_size" >> collectd.conf
        fi
    else
        echo -e "           port \"$hec_port\"" >> collectd.conf
        echo -e "           token \"$hec_token\"" >> collectd.conf
        echo -e "           ssl true" >> collectd.conf
        echo -e "           verifyssl false" >> collectd.conf
    fi
    echo -e "           Dimension \"entity_type:nix_host\"" >> collectd.conf
    for i in "${dimensions[@]}"
    do
        echo -e "           Dimension \"$i\"" >> collectd.conf
    done
    if [[ $CLOUD = "AWS" ]]; then
        if [ -n "$aws_instance_id" ] && [ -n "$aws_account_id" ]; then
            echo -e "           Dimension \"InstanceId:$aws_instance_id\"" >> collectd.conf
            echo -e "           Dimension \"AccountID:$aws_account_id\"" >> collectd.conf
            echo -e "           Dimension \"Cloud:AWS\"" >> collectd.conf
        else
            echo "ERROR: Please configure Cloud Type, AWS instance ID and AWS Account ID manually in collectd.conf under <Plugin write_splunk> as follows: "
            echo "Dimension \"InstanceId:<instance_id>\""
            echo "Dimension \"AccountID:<account_id>\""
            echo "Dimension \"Cloud:AWS\""
            echo "Note: Please configure each dimension in a new line."
        fi
    elif [ "$OS" = "EC2" ]; then
        ec2_instance_id=$(ec2-metadata -i | cut -d ":" -f2 | tr -d ' ')
        echo -e "           Dimension \"InstanceId:$ec2_instance_id\"" >> collectd.conf
    elif [[ $CLOUD = "Azure" ]]; then
        if [ -n "$azure_resource_id" ] && [ -n "$azure_subscription_id" ]; then
            lowercaseresourceid=$(echo $azure_resource_id | tr '[:upper:]' '[:lower:]')
            echo -e "           Dimension \"ResourceId:$azure_resource_id\"" >> collectd.conf
            echo -e "           Dimension \"LowerCaseResourceId:$lowercaseresourceid\"" >> collectd.conf
            echo -e "           Dimension \"SubscriptionId:$azure_subscription_id\"" >> collectd.conf
            echo -e "           Dimension \"Cloud:Azure\"" >> collectd.conf
        else
            echo "ERROR: Please configure Cloud Type, Azure Resource ID, Lower case Azure Resource ID and Azure Subscription ID manually in collectd.conf under <Plugin write_splunk> as follows: "
            echo "Dimension \"ResourceId:<resource_id>\""
            echo "Dimension \"LowerCaseResourceId:<resource_id in all lower case>\""
            echo "Dimension \"SubscriptionId:<subscription_id>\""
            echo "Dimension \"Cloud:Azure\""
            echo "Note: Please configure each dimension in a new line."
        fi
    fi
    echo -e "</Plugin>" >> collectd.conf
    get_splunk_hostname

    sed_inplace "s/#Hostname    \"collectd.server.sample\"/Hostname    \"$splunk_hostname\"/g" "collectd.conf"
    if [ -n "$enable_docker" ]; then
      echo -e "\nLoadplugin docker" >> collectd.conf
      echo -e "<Plugin docker>" >> collectd.conf
      echo -e "           dockersock \"$docker_unix_sock\"" >> collectd.conf
      echo -e "           apiversion \"v1.20\"" >> collectd.conf
      echo -e "</Plugin>" >> collectd.conf
    fi
}

function download_deps_debian {
    print_info "Step:Downloading dependencies..."

    $_sudo apt-get install -y libyajl2

    # install libcurl dependency
    if [ -n "$CLOUD" ]; then
        #TODO This may conflict with libcurl3 if it's already installed. Needs to be tested more
        $_sudo apt-get install -y curl
    fi
        
    debian_libcurl3=false
    debian_libcurl4=false
    # Check if libcurl3 already installed
    if apt-cache policy libcurl3 | grep -i "installed" | grep -vi "none"; then
        print_info "libcurl3 already installed. Continuing .."
        debian_libcurl3=true
        return
    fi

    # Check if libcurl4 already installed
    if apt-cache policy libcurl4 | grep -i "installed" | grep -vi "none"; then
        print_info "libcurl4 already installed. Continuing .."
        debian_libcurl4=true
        return
    fi

    # Check if libcurl4 available, install if yes
    if apt-cache policy libcurl4 | grep -i "candidate" | grep -vi "none"; then
        print_info "installing libcurl4 .."
        $_sudo apt-get install -y libcurl4
        debian_libcurl4=true
        return
    fi

    # Check if libcurl3 available, install if yes
    if apt-cache policy libcurl3 | grep -i "candidate" | grep -vi "none"; then
        print_info "installing libcurl3 .."
        $_sudo apt-get install -y libcurl3
        debian_libcurl3=true
    else
        print_warning "Failed to install libcurl package. exiting .."
        exit 1
    fi
    
}

function download_deps_suse {
    zypper search --installed-only libcurl
    if [ $? -eq 0 ] ; then
        # libcurl package already installed
        return
    fi

    print_info "Step:Downloading dependencies..."

    # install libcurl dependency
    $_sudo zypper install -y libcurl4
    $_sudo zypper install -y libyajl2
}

function add_collectd_source_debian {
    debian_distribution=$(cat /etc/os-release | grep -m1 -iEo 'wheezy|jessie|trusty|xenial' | tr '[:upper:]' '[:lower:]' | head -1)
    if [ -z "$debian_distribution" ]; then
        print_warning "Unsupported Debian version..."
        exit 1
    fi

    # import repo signing key if add_key enabled
    if [[ $authenticated_install = "Yes" ]]; then
        wget -qO - https://pkg.ci.collectd.org/pubkey.asc | $_sudo apt-key add -
    fi

    # Enforce apt-get to install collected-5.8.*
    if ! grep -Fxq "deb http://pkg.ci.collectd.org/deb ${debian_distribution} collectd-5.8" /etc/apt/sources.list.d/collectd-ci.list; then
        echo "deb http://pkg.ci.collectd.org/deb ${debian_distribution} collectd-5.8" | $_sudo tee -a /etc/apt/sources.list.d/collectd-ci.list
        $_sudo apt-get update
    fi
}

function download_deps_redhat {
    print_info "Step:Downloading dependencies..."

    # install libcurl and libyajl dependencies
    $_sudo yum install -y libcurl
    $_sudo yum install -y yajl
}

function download_deps_solaris {
    print_info "Step:Downloading dependencies..."

    # install libcurl dependency
    $_sudo yes|pkgadd -d http://get.opencsw.org/now all
    $_sudo /opt/csw/bin/pkgutil -U
    $_sudo /opt/csw/bin/pkgutil -y -i libcurl4_feature
}

function install_collectd_suse {
    if hash collectd 2>/dev/null; then
        $_sudo service collectd stop

        # check for collectd version 5.7.* - 5.12.*
        collectd_version=$(zypper -n search -s --installed-only collectd | grep -Eo "$COLLECTD_VERSION")

        # if collectd 5.[7-12].x not installed, error out
        if [ -z "$collectd_version" ]; then
            print_warning "$UNSUPP_COLLECTD_INSTALLED_MSG"
            exit 1
        fi

        # download any missing dependencies
        download_deps_suse
        # Backup collectd.conf
        $_sudo mv /etc/collectd.conf "/etc/collectd/collectd.conf.old.$(date +'%Y%m%d-%H%M%S')"
        return
    fi

    # Extract SUSE version
    version=$(grep "^VERSION_ID" /etc/os-release | grep -Eo '".*"' | tr -d \")

    os_ver="${OS}${version}"

    case $os_ver in
        OpenSuse15*)
            : # do nothing, required collectd package available for install with zypper
            ;;
        EnterpriseSuse15*)
            $_sudo zypper -n addrepo 'https://download.opensuse.org/repositories/server:monitoring/SLE_15/server:monitoring.repo'
            $_sudo zypper -n --gpg-auto-import-keys refresh 'server_monitoring'
            $_sudo zypper -n addrepo 'https://download.opensuse.org/repositories/openSUSE:/Backports:/SLE-15/standard/openSUSE:Backports:SLE-15.repo'
            $_sudo zypper -n --gpg-auto-import-keys refresh 'openSUSE_Backports_SLE-15'
            ;;
        *)
            print_warning "Unsupported SUSE version. Supported versions are SUSE Enterprise 15 & openSUSE 15"
            exit 1
    esac

    collectd_version=$(zypper -n info collectd | grep -Eo "$COLLECTD_VERSION")

    # if collectd version 5.7.* - 5.12.* not available
    if [ -z "$collectd_version" ]; then
        print_warning "$UNSUPP_COLLECTD_AVAILABLE_MSG"
        exit 1
    fi

    # Install any missing dependency packages
    download_deps_suse

    # Install collectd
    $_sudo zypper install -y collectd
    if [ $? -ne 0 ] ; then
        print_warning "Failed to install collectd"
        exit 1
    fi
}

function install_collectd_debian {
    # Download dependency
    download_deps_debian

    print_info "Step:Installing collectd on Debian..."

    if hash collectd 2>/dev/null; then
        $_sudo service collectd stop

        # check for collectd version 5.7.* - 5.12.*
        collectd_version=$(dpkg -l | grep "collectd" | grep -Eo "$COLLECTD_VERSION")

        # if collectd 5.[7-12].x not installed
        if [ -z "$collectd_version" ]; then
            print_warning "$UNSUPP_COLLECTD_INSTALLED_MSG"
            exit 1
        fi

        # Backup
        $_sudo mv /etc/collectd/collectd.conf "/etc/collectd/collectd.conf.old.$(date +'%Y%m%d-%H%M%S')"
    else
        # first check collectd version for install candidate
        $_sudo apt-get update
        collectd_version=$(apt-cache policy collectd | grep -i 'candidate' | grep -Eo "$COLLECTD_VERSION")

        if [ -z "$collectd_version" ]; then
            # add collectd source and install collectd
            add_collectd_source_debian
            collectd_version='5.8'
            if [[ $authenticated_install = "Yes" ]]; then
                $_sudo apt-get --yes install collectd
            else
                $_sudo apt-get --yes --allow-unauthenticated install collectd
            fi
        else
            # if collectd verison is available
            # avoid using --allow-unauthenticated even user choose unauthenticated
            $_sudo apt-get --yes install collectd
        fi

        # Install missing dependencies
        print_info "Installing any missing dependency..."
        $_sudo apt-get -f install
    fi
}

function install_collectd_solaris {
    # Download dependency
    download_deps_solaris

    print_info "Step:Installing collectd on Solaris..."

    if hash collectd 2>/dev/null; then
        $_sudo svcadm disable cswcollectd

        # check for collectd version 5.7.* - 5.12.* for Solaris
        collectd_version=$(/opt/csw/bin/pkgutil -a CSWcollectd | grep 'collectd' | head -1 | sed 's/.*5\./5\./g')

        # if collectd not installed, exit
        if [ -z "$collectd_version" ]; then
            print_warning "$UNSUPP_COLLECTD_INSTALLED_MSG"
            exit 1
        fi

        # Backup
        $_sudo mv /etc/opt/csw/collectd.conf "/etc/opt/csw/collectd.conf.old.$(date +'%Y%m%d-%H%M%S')"
    else
        collectd_version=$(/opt/csw/bin/pkgutil -a CSWcollectd | grep 'collectd' | head -1 | sed 's/.*5\./5\./g')
        # install collectd package for solaris
        $_sudo /opt/csw/bin/pkgutil -y -i collectd
    fi
}

function install_collectd_redhat {
    RHEL_8_VERSION="^4.18.0(.)+"
    RHEL_7_VERSION="^3.10.0(.)+"
    RHEL_6_VERSION="^2.6.32(.)+"

    print_info "Step:Installing collectd on Redhat..."

    is_rhel_6=$(uname -r | grep -Eo "$RHEL_6_VERSION")
    is_rhel_7=$(uname -r | grep -Eo "$RHEL_7_VERSION")
    is_rhel_8=$(uname -r | grep -Eo "$RHEL_8_VERSION")

    is_centos_6=false
    is_centos_8=false
    is_fedora=false
    if [[ -f /etc/centos-release && $( cat /etc/centos-release | grep -oP "[0-9]+" | head -1 ) -eq 6 ]]; then
        is_centos_6=true
    fi

    if [[ -f /etc/centos-release && $( cat /etc/centos-release | grep -oP "[0-9]+" | head -1 ) -eq 8 ]]; then
        is_centos_8=true
    fi

    if [[ -f /etc/fedora-release ]]; then
        is_fedora=true
    fi

    if [ "$is_centos_6" = true ] || [ "$is_rhel_6" ] || [ "$is_fedora" = true ]; then
        # if Centos6 or RHEL6
        :
    elif [ "$is_rhel_8" ]; then
        $_sudo yum install -y https://dl.fedoraproject.org/pub/epel/epel-release-latest-8.noarch.rpm
    elif [ "$is_rhel_7" ]; then
        # This install statement is taken from https://aws.amazon.com/premiumsupport/knowledge-center/ec2-enable-epel/
        $_sudo yum install -y https://dl.fedoraproject.org/pub/epel/epel-release-latest-7.noarch.rpm
    else
        $_sudo yum -y install epel-release
    fi

    # Install any missing dependency packages
    download_deps_redhat

    if hash collectd 2>/dev/null; then
        $_sudo service collectd stop
        # Backup
        $_sudo mv /etc/collectd.conf "/etc/collectd.conf.old.$(date +'%Y%m%d-%H%M%S')"
        # check for collectd version 5.[7-10].x
        collectd_version=$(yum list installed | grep "collectd" | grep -Eo "$COLLECTD_VERSION")
        if [ -n "$collectd_version" ]; then
            # correct version already installed..
            return 0
        else
            print_warning "$UNSUPP_COLLECTD_INSTALLED_MSG"
            exit 1
        fi
    fi

    if [ "$is_centos_6" = true ] || [ "$is_rhel_6" ]; then
        # extract latest collectd rpm build number
        wget https://pkg.ci.collectd.org/rpm/collectd-5.7/epel-6-x86_64/status.json
        build_num=$(cat status.json | grep -Eo '5.7\.[^"]*')

        if [ $? -ne 0 ] ; then
            print_warning "Failed to find the collectd package"
            exit 1
        fi
        collectd_version='5.7'
        # install collectd and disk rpm
        $_sudo yum install -y "https://pkg.ci.collectd.org/rpm/collectd-5.7/epel-6-x86_64/collectd-${build_num}-3.el6.x86_64.rpm"
        $_sudo yum install -y "https://pkg.ci.collectd.org/rpm/collectd-5.7/epel-6-x86_64/collectd-disk-${build_num}-3.el6.x86_64.rpm"
    elif [ "$is_fedora" = true ]; then
         collectd_version=$(yum list available collectd | grep -Eo "$COLLECTD_VERSION")
        if [ -n "$collectd_version" ]; then
            # Install collectd
            $_sudo yum -y install collectd
        else
            print_warning "$UNSUPP_COLLECTD_AVAILABLE_MSG"
            exit 1
        fi
    else
        # check if the required collectd version 5.[7-12].x is available
        collectd_version=$(yum --enablerepo=epel list available collectd | grep -Eo "$COLLECTD_VERSION")
        if [ -n "$collectd_version" ]; then
            # Install collectd
            $_sudo yum -y --enablerepo=epel install collectd
        else
            print_warning "$UNSUPP_COLLECTD_AVAILABLE_MSG"
            exit 1
        fi
    fi

    # collectd's disk plugin is in seperate package
    if [ "$OS" = "EC2" ] || [ "$is_fedora" = true ] || [ "$is_centos_8" = true ] || [ "$is_rhel_8" ]; then
        yum -y install collectd-disk
    fi

}

# refers to brew command for prefix (if installed),
# otherwise will return default location where brew is usually installed to
function get_brew_prefix {
    brew --prefix 2>/dev/null || echo '/usr/local'
}

function install_collectd_darwin {
    print_info "Step:Installing collectd on Darwin..."
    current_working_dir=$(pwd)

    brew_prefix=$(get_brew_prefix)
    brew_sbin="$brew_prefix/sbin"

    # Check if brew is installed
    which -s brew
    if [[ $? != 0 ]] ; then
      print_warning "Homebrew is not installed. Please install Homebrew ..."
      exit 1
    fi

    cd $current_working_dir

    # Check if collectd is installed
    if brew ls --versions collectd > /dev/null; then
        # Check if installed version meets requirements
        collectd_version=$(brew ls --versions collectd | grep -Eo "$COLLECTD_VERSION_DARWIN" | head -1)
        if [ -z "$collectd_version" ]; then
          print_warning "$UNSUPP_COLLECTD_INSTALLED_MSG"
          print_warning "Remove unsupported version using 'brew remove collectd' and rerun the script ..."
          exit 1
        fi
        # Backup
        $_sudo mv "$brew_prefix/Cellar/collectd/${collectd_version}/etc/collectd.conf" "$brew_prefix/Cellar/collectd/${collectd_version}/etc/collectd.conf.old.$(date +'%Y%m%d-%H%M%S')"
    else
        # Check collectd version on homebrew
        current_version_on_brew=$(brew info collectd | grep "collectd: " | grep -Eo "$COLLECTD_VERSION_DARWIN")
        if [ -z "$current_version_on_brew" ]; then
          print_warning "$UNSUPP_COLLECTD_AVAILABLE_MSG"
          exit 1
        fi
        # This directory is required to use brew to download collectd
        if [ ! -d "$brew_sbin" ]; then
          print_info "Required directory $brew_sbin not found. Creating $brew_sbin now."
          $_sudo mkdir -p "$brew_sbin"
          $_sudo chown -R $(whoami) "$brew_sbin"
        fi

        brew install collectd
        if [ $? -ne 0 ]; then
          print_warning "Failed to install supported version of collectd ..."
          exit 1
        fi
        collectd_version=$(brew ls --versions collectd | grep -Eo "$COLLECTD_VERSION_DARWIN" | head -1)
    fi
}

function setAttributesByCloud {
    if [[ $CLOUD = "AWS" ]]; then
        # Please refer to AWS Docs: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-metadata.html
        # to see how we can get AWS EC2 metadata information.
        aws_instance_id=$(eval curl -fsL --max-time 3 --retry 3 http://169.254.169.254/latest/meta-data/instance-id)
        if [ $? -ne 0 ]; then
            echo "Failed to get AWS EC2 Instance ID"
        fi

		aws_account_id=$(eval curl -fsL --max-time 3 --retry 3 http://169.254.169.254/latest/meta-data/identity-credentials/ec2/info/ | grep AccountId | grep -o [0-9]*)
        if [ $? -ne 0 ]; then
            echo "Failed to get AWS Account ID"
        fi
    elif [[ $CLOUD = "Azure" ]]; then
        azure_resource_id=$(curl -fsL --max-time 3 --retry 3 -H Metadata:true "http://169.254.169.254/metadata/instance/compute/resourceId?api-version=2019-06-01&format=text")
        if [ $? -ne 0 ]; then
            echo "Failed to get Azure VM ID"
        fi

        azure_subscription_id=$(curl -fsL --max-time 3 --retry 3 -H Metadata:true "http://169.254.169.254/metadata/instance/compute/subscriptionId?api-version=2019-06-01&format=text")
        if [ $? -ne 0 ]; then
            echo "Failed to get Azure Subscription ID"
        fi
    fi
    #GCP
}

function install_yajl_darwin {
  installed_version=$(brew ls --versions yajl)
  if [ -z "$installed_version" ]; then
    print_info "installing yajl"
    brew install yajl
  else
    print_info "yajl already installed, skipping..."
  fi
}

if [ "$OS" = "Debian" ]; then
    print_info "Installing Agents on Debian"
    check_selinux "$@"
    install_collectd_debian
    configure
elif [ "$OS" = "RedHat" ]; then
    print_info "Installing Agents on RedHat"
    check_selinux "$@"
    install_collectd_redhat
    enable_collectd
    configure
elif [ "$OS" = "Darwin" ]; then
    print_info "Installing Agents on Darwin"
    if [ -n "$enable_docker" ]; then
      install_yajl_darwin
    fi
    install_collectd_darwin
    configure
elif [ "$OS" = "EC2" ]; then
    print_info "Installing Agents on EC2 instance"
    check_selinux "$@"
    install_collectd_redhat
    configure
elif [ "$OS" = "OpenSuse" ]; then
    print_info "Installing Agents on openSUSE"
    check_selinux "$@"
    install_collectd_suse
    configure
elif [ "$OS" = "EnterpriseSuse" ]; then
    print_info "Installing Agents on SUSE Enterprise"
    check_selinux "$@"
    install_collectd_suse
    configure
elif [ "$OS" = "SunOS" ]; then
    print_info "Installing Agents on Solaris"
    install_collectd_solaris
    configure
else
    print_warning "Not supported operating system: $OS. Nothing is installed."
fi

